@@ -222,8 +222,10 @@ sub _bundle_ref_path_expand {
 
 sub _coerce_arrays {
   my ($self, $val, $param) = @_;
-  my $data_type   = data_type $val->{value};
-  my $schema_type = schema_type $param->{schema} || $param;    # $param->{schema} is for OpenAPIv3
+  my $data_type = data_type $val->{value};
+  my $schema    = $param->{schema} || $param;    # $param->{schema} is for OpenAPIv3
+  $schema = $self->get($schema->{'$ref'} =~ s!^#!!r) if $schema->{'$ref'};
+  my $schema_type = schema_type $schema;
   return $val->{value} = [$val->{value}] if $schema_type eq 'array' and $data_type ne 'array';
   return $val->{value} = @{$val->{value}} ? $val->{value}[-1] : undef
     if $schema_type ne 'array' and $data_type eq 'array';
