@@ -1,17 +1,23 @@
 package JSON::Validator::Store;
 use Mojo::Base -base;
 
-use Mojo::Exception qw(raise);
+use Mojo::Exception;
 use Mojo::File qw(path);
 use Mojo::JSON;
+use Mojo::JSON::Pointer;
 use Mojo::UserAgent;
 use Mojo::Util qw(url_unescape);
 use JSON::Validator::Schema;
-use JSON::Validator::Util qw(data_section);
+use JSON::Validator::URI qw(uri);
+use JSON::Validator::Util qw(data_section str2data);
+use Scalar::Util qw(blessed);
 
+use constant DEBUG         => $ENV{JSON_VALIDATOR_DEBUG} && 1;
 use constant BUNDLED_PATH  => path(path(__FILE__)->dirname, 'cache')->to_string;
 use constant CASE_TOLERANT => File::Spec->case_tolerant;
 
+die $@ unless eval q(package JSON::Validator::Exception; use Mojo::Base 'Mojo::Exception'; 1);
+
 has cache_paths => sub { [split(/:/, $ENV{JSON_VALIDATOR_CACHE_PATH} || ''), BUNDLED_PATH] };
 has schemas     => sub { +{} };
 
@@ -50,32 +56,91 @@ sub load {
     || $_[0]->_load_from_file($_[1])
     || $_[0]->_load_from_app($_[1])
     || $_[0]->get($_[1])
-    || raise 'JSON::Validator::Exception', "Unable to load schema $_[1]";
+    || _raise(qq(Unable to load schema "$_[1]".));
+}
+
+sub resolve {
+  my ($self, $ref, $curr) = @_;
+  $curr //= {base_url => ''};
+
+  my ($base_url, $fragment) = split '#', $ref;
+  my $abs_url = uri($base_url)->fragment($fragment);
+  $abs_url  = uri $abs_url, $curr->{base_url} if $curr->{base_url} and !$abs_url->is_abs;
+  $fragment = '' unless defined $fragment;
+  $base_url ||= $curr->{base_url} || '';
+
+  warn "[JSON::Validator] Resolve curr: ref=$ref,@{[map qq($_=$curr->{$_}), sort keys %$curr]}\n" if DEBUG;
+
+  my $state = {base_url => $base_url, fragment => $fragment, source => 'unknown'};
+  if (defined(my $schema = $self->schemas->{$abs_url})) {
+    @$state{qw(base_url id root schema source)} = ($abs_url, $abs_url, $schema, $schema, 'schema/abs_url');
+  }
+  elsif (defined(my $root = $self->schemas->{$base_url})) {
+    @$state{qw(base_url id root source)} = ($base_url, $base_url, $root, 'schema/base_url');
+  }
+  elsif ($base_url) {
+    $base_url = uri $base_url, $curr->{base_url} if $curr->{base_url};
+    my $id = $self->load($base_url);
+    @$state{qw(base_url id root source)} = ($id, $id, $self->get($id), 'load');
+    $state->{root} = $self->get($id);
+  }
+  else {
+    @$state{qw(id root source)} = ('', $curr->{root}, 'root');
+  }
+
+  $fragment =~ s!%2f!~1!;    # /
+  $fragment =~ s!%7e!~0!;    # ~
+  $fragment = url_unescape $fragment;
+  $state->{schema} //= length $fragment ? Mojo::JSON::Pointer->new($state->{root})->get($fragment) : $state->{root};
+  _raise(qq[Unable to resolve "$ref" from "$state->{base_url}". ($state->{source})]) unless defined $state->{schema};
+
+  $state->{$_} //= $curr->{$_} for keys %$curr;    # pass on original information
+  warn "[JSON::Validator] Resolve state: @{[map qq($_=$state->{$_}), sort keys %$state]}\n" if DEBUG;
+  return $state;
+}
+
+sub _add {
+  my ($self, $id, $schema) = @_;
+  $id = $self->add($id => $schema);
+
+  if (ref $schema eq 'HASH') {
+    return
+        $schema->{'$id'} ? $self->add($schema->{'$id'} => $schema)
+      : $schema->{id}    ? $self->add($schema->{id} => $schema)
+      :                    $id;
+  }
+
+  return $id;
 }
 
 sub _load_from_app {
   return undef unless $_[1] =~ m!^/!;
 
-  my ($self, $url, $id) = @_;
+  my ($self, $url) = @_;
+  my $id;
   return undef unless $self->ua->server->app;
+  return undef if blessed $url and !$url->can('scheme');
   return $id if $id = $self->exists($url);
 
   my $tx  = $self->ua->get($url);
   my $err = $tx->error && $tx->error->{message};
-  raise 'JSON::Validator::Exception', $err if $err;
-  return $self->add($url => _parse($tx->res->body));
+  _raise("GET $url: $err")                      if $err;
+  warn "[JSON::Validator] Load from app $url\n" if DEBUG;
+  return $self->_add($url => str2data $tx->res->body);
 }
 
 sub _load_from_data {
   return undef unless $_[1] =~ m!^data://([^/]*)/(.*)!;
 
-  my ($self, $url, $id) = @_;
+  my ($self, $url) = @_;
+  my $id;
   return $id if $id = $self->exists($url);
 
   my ($class, $file) = ($1, $2);    # data://([^/]*)/(.*)
   my $text = data_section $class, $file, {encoding => 'UTF-8'};
-  raise 'JSON::Validator::Exception', "Could not find $url" unless $text;
-  return $self->add($url => _parse($text));
+  _raise("Could not find $url") unless $text;
+  warn "[JSON::Validator] Load from data $file in $class\n" if DEBUG;
+  return $self->_add($url => str2data $text);
 }
 
 sub _load_from_file {
@@ -87,8 +152,9 @@ sub _load_from_file {
   return undef unless -e $file;
 
   $file = $file->realpath;
-  my $id = Mojo::URL->new->scheme('file')->host('')->path(CASE_TOLERANT ? lc $file : "$file");
-  return $self->exists($id) || $self->add($id => _parse($file->slurp));
+  my $id = uri()->new->scheme('file')->host('')->path(CASE_TOLERANT ? lc $file : "$file");
+  warn "[JSON::Validator] Load from file $file\n" if DEBUG;
+  return $self->exists($id) || $self->_add($id => str2data $file->slurp);
 }
 
 sub _load_from_text {
@@ -96,42 +162,43 @@ sub _load_from_text {
   my $is_scalar_ref = ref $text eq 'SCALAR';
   return undef unless $is_scalar_ref or $text =~ m!^\s*(?:---|\{)!s;
 
-  my $id = sprintf 'urn:text:%s', Mojo::Util::md5_sum($is_scalar_ref ? $$text : $text);
-  return $self->exists($id) || $self->add($id => _parse($is_scalar_ref ? $$text : $text));
+  my $id = uri->from_data($is_scalar_ref ? $$text : $text);
+  warn "[JSON::Validator] Load from text $id\n" if DEBUG;
+  return $self->exists($id) || $self->_add($id => str2data $is_scalar_ref ? $$text : $text);
 }
 
 sub _load_from_url {
   return undef unless $_[1] =~ m!^https?://!;
 
-  my ($self, $url, $id) = @_;
+  my ($self, $url) = @_;
+  my $id;
   return $id if $id = $self->exists($url);
 
-  $url = Mojo::URL->new($url)->fragment(undef);
+  $url = uri($url)->fragment(undef);
   return $id if $id = $self->exists($url);
 
   my $cache_path = $self->cache_paths->[0];
   my $cache_file = Mojo::Util::md5_sum("$url");
   for (@{$self->cache_paths}) {
     my $path = path $_, $cache_file;
-    return $self->add($url => _parse($path->slurp)) if -r $path;
+    warn "[JSON::Validator] Load from cache $path\n"  if DEBUG and -r $path;
+    return $self->_add($url => str2data $path->slurp) if -r $path;
   }
 
   my $tx  = $self->ua->get($url);
   my $err = $tx->error && $tx->error->{message};
-  raise 'JSON::Validator::Exception', $err if $err;
+  _raise("GET $url: $err") if $err;
 
   if ($cache_path and $cache_path ne BUNDLED_PATH and -w $cache_path) {
     $cache_file = path $cache_path, $cache_file;
     $cache_file->spurt($tx->res->body);
   }
 
-  return $self->add($url => _parse($tx->res->body));
+  warn "[JSON::Validator] Load from URL $url\n" if DEBUG;
+  return $self->_add($url => str2data $tx->res->body);
 }
 
-sub _parse {
-  return Mojo::JSON::decode_json($_[0]) if $_[0] =~ m!^\s*\{!s;
-  return JSON::Validator::Util::_yaml_load($_[0]);
-}
+sub _raise { die JSON::Validator::Exception->new(@_)->trace }
 
 1;
 
@@ -233,6 +300,23 @@ Loading can also be done with relative path, which will then load from:
 
 This method is EXPERIMENTAL, but unlikely to change significantly.
 
+=head2 resolve
+
+  $hash_ref = $store->resolve($url, \%defaults);
+
+Takes a C<$url> (can also be a file, urn, ...) with or without a fragment and
+returns this structure about the schema:
+
+  {
+    base_url => $str,  # the part before the fragment in the $url
+    fragment => $str,  # fragment part of the $url
+    id       => $str,  # store ID
+    root     => ...,   # the root schema
+    schema   => ...,   # the schema inside "root" if fragment is present
+  }
+
+This method is EXPERIMENTAL and can change without warning.
+
 =head1 SEE ALSO
 
 L<JSON::Validator>.
